#ifndef _I2CDEVICECLASS_H
#define _I2CDEVICECLASS_H

//#ifdef _WIN32_WCE

#include "i2cmgr.h"


// Define the following to turn I/O debug msgs on.
// #define DEBUG_IO


// NOTE to TOM - if this inline dont work use the #define

_inline BYTE BitInByte(BYTE Index){return (static_cast<BYTE>(0x01 << Index));};
//#define BitInByte(Index) ( (BYTE)(0x01 << Index) )
// converts value of Index to a byte with one bit set high
// where the high bit is bit Index of the byte
// eg if Index == 0 BitInByte returns 0x01 which is binary 0000 0001
// eg if Index == 1 BitInByte returns 0x02 which is binary 0000 0010
// eg if Index == 2 BitInByte returns 0x04 which is binary 0000 0100
// eg if Index == 3 BitInByte returns 0x08 which is binary 0000 1000
// eg if Index == 4 BitInByte returns 0x10 which is binary 0001 0000
// eg if Index == 5 BitInByte returns 0x20 which is binary 0010 0000
// eg if Index == 6 BitInByte returns 0x40 which is binary 0100 0000
// eg if Index == 7 BitInByte returns 0x80 which is binary 1000 0000
// what this means is that, on the IO board, bit 0 of a Mask register
// corresponds to Alarm0, Filter0, Port0, etc.




// IODEVICE OPCODES
#define TEST_OPCODE                     0x00
#define RESET_OPCODE                    0x01
#define SENDUNITID_OPCODE               0x02
#define READTYPE_OPCODE                 0x03
#define CHANGEADDRESS_OPCODE            0x04
#define RUN_OPCODE                      0x05
#define STOP_OPCODE                     0x06
#define READREGISTER_OPCODE             0x07
#define WRITEREGISTER_OPCODE            0x08
#define CHANGEDESTINATIONADDRESS_OPCODE 0x0D
#define CLEARSTATUS_OPCODE              0x0E
#define WRITEMODULENO_OPCODE            0x21

// DIGITAL IO OPCODES
#define CONFIGURE_OPCODE                0x09
#define READDATALO_OPCODE               0x0A
#define LOADCOUNTER_OPCODE              0x0B
#define CONFIGUREFILTERS_OPCODE         0x0C
#define SETSENSITIVITY_OPCODE           0x0F
#define AUXON_OPCODE                    0x10
#define AUXOFF_OPCODE                   0x11
#define SETSPEED_OPCODE                 0x12
#define PULSEOUT_OPCODE                 0x13
#define READDATAHI_OPCODE               0x14
#define ALARM_OPCODE                    0x90    // this is a returned opcode

// ANALOG IO OPCODES
#define CALIBRATE_OPCODE                0x09
// PULSEOUT_OPCODE same as digital = 0x13
#define SETAUX1_OPCODE                  0x14
#define MEASUREVOLTAGE_OPCODE           0x0A
#define MEASURECURRENT_OPCODE           0x0B
#define MEASURE420MA_OPCODE             0x0C
#define READRMYOUNGCOUNTER_OPCODE       0x19
#define MEASURERESISTANCEDC_OPCODE      0x1A
#define MEASURERESISTANCEAC_OPCODE      0x1B
#define MEASURETHERMISTOR_OPCODE        0x1C
#define READCALIBRATIONDATA_OPCODE      0x1D
#define READCALIBRATIONDATAHI_OPCODE    0x20
#define MEASUREVOLTAGEDOUBLE_OPCODE     0x18

//DISPLAY IO OPCODES
#define GETKEYSTATUS_OPCODE         0x2D
#define KEYSTATUS_OPCODE            0x6D
#define KEYPRESSED_OPCODE           0x6E
#define SHOWCURSOR_OPCODE           0x34
#define HIDECURSOR_OPCODE           0x35
#define STARTBLINKINGCURSOR_OPCODE  0x36
#define STOPBLINKINGCURSOR_OPCODE   0x37
#define WRSTRINGTOLCD_OPCODE        0x38
#define CLEARDISPLAY_OPCODE         0x3A


// NOTE: return opcode = send opcode | 0x40


// IODEVICE OPCODE Strings
static const TCHAR *  UNKNOWN_OPCODE_STR                    = {_T("Unknown")};
static const TCHAR *  TEST_OPCODE_STR                       = {_T("Test")};
static const TCHAR *  RESET_OPCODE_STR                      = {_T("Reset")};
static const TCHAR *  SENDUNITID_OPCODE_STR                 = {_T("Send Unit ID")};
static const TCHAR *  READTYPE_OPCODE_STR                   = {_T("Read Type")};
static const TCHAR *  CHANGEADDRESS_OPCODE_STR              = {_T("Change Address")};
static const TCHAR *  RUN_OPCODE_STR                        = {_T("Run")};
static const TCHAR *  STOP_OPCODE_STR                       = {_T("Stop")};
static const TCHAR *  READREGISTER_OPCODE_STR               = {_T("Read Register")};
static const TCHAR *  WRITEREGISTER_OPCODE_STR              = {_T("Write Register")};
static const TCHAR *  CHANGEDESTINATIONADDRESS_OPCODE_STR   = {_T("Change Destination Address")};
static const TCHAR *  CLEARSTATUS_OPCODE_STR                = {_T("Clear Status")};
static const TCHAR *  WRITEMODULENO_OPCODE_STR              = {_T("Write Module Number")};

// DIGITAL IO OPCODE Strings
static const TCHAR *  CONFIGURE_OPCODE_STR                  = {_T("Configure")};
static const TCHAR *  READDATALO_OPCODE_STR                 = {_T("Read Data Lo")};
static const TCHAR *  READDATAHI_OPCODE_STR                 = {_T("Read Data Hi")};
static const TCHAR *  LOADCOUNTER_OPCODE_STR                = {_T("Load Counter")};
static const TCHAR *  CONFIGUREFILTERS_OPCODE_STR           = {_T("Configure Filters")};
static const TCHAR *  SETSENSITIVITY_OPCODE_STR             = {_T("Set Sensitivity")};
static const TCHAR *  AUXON_OPCODE_STR                      = {_T("Aux On")};
static const TCHAR *  AUXOFF_OPCODE_STR                     = {_T("Aux Off")};
static const TCHAR *  SETSPEED_OPCODE_STR                   = {_T("Set Speed")};
static const TCHAR *  PULSEOUT_OPCODE_STR                   = {_T("Pulse Out")};
static const TCHAR *  ALARM_OPCODE_STR                      = {_T("Alarm")};

// ANALOG IO OPCODE Strings
static const TCHAR *  CALIBRATE_OPCODE_STR                  = {_T("Calibrate")};
// PULSEOUT_OPCODE same as digital = 0x13
static const TCHAR *  SETAUX1_OPCODE_STR                    = {_T("Set Aux1")};
static const TCHAR *  MEASUREVOLTAGE_OPCODE_STR             = {_T("Measure Voltage")};
static const TCHAR *  MEASURECURRENT_OPCODE_STR             = {_T("Measure Current")};
static const TCHAR *  MEASURE420MA_OPCODE_STR               = {_T("Measure 4-20 ma")};
static const TCHAR *  READRMYOUNGCOUNTER_OPCODE_STR         = {_T("Read RMYoung Counter")};
static const TCHAR *  MEASURERESISTANCEDC_OPCODE_STR        = {_T("Measure Resistance DC")};
static const TCHAR *  MEASURERESISTANCEAC_OPCODE_STR        = {_T("Measure Resistance AC")};
static const TCHAR *  MEASURETHERMISTOR_OPCODE_STR          = {_T("Measure Thermistor")};
static const TCHAR *  READCALIBRATIONDATA_OPCODE_STR        = {_T("Read Calibration Data")};
static const TCHAR *  READCALIBRATIONDATAHI_OPCODE_STR      = {_T("Read HiGain Calibration Data")};
static const TCHAR *  MEASUREVOLTAGEDOUBLE_STR              = {_T("Measure Voltage Double")};

// DISPLAY IO OPCODE Strings
static const TCHAR *  WRSTRINGTOLCD_OPCODE_STR              = {_T("Write string to display")};
static const TCHAR *  KEYPRESSED_OPCODE_STR                 = {_T("Keypressed")};
static const TCHAR *  SHOWCURSOR_OPCODE_STR                 = {_T("Show cursor")};
static const TCHAR *  HIDECURSOR_OPCODE_STR                 = {_T("Hide cursor")};
static const TCHAR *  STARTBLINKINGCURSOR_OPCODE_STR        = {_T("Blink Cursor")};
static const TCHAR *  STOPBLINKINGCURSOR_OPCODE_STR         = {_T("Stop blink cursor")};
static const TCHAR *  CLEARDISPLAY_OPCODE_STR               = {_T("Clear display")};
static const TCHAR *  GETKEYSTATUS_OPCODE_STR               = {_T("Get key status")};
static const TCHAR *  KEYSTATUS_OPCODE_STR                  = {_T("Key status")};


/**********************************************************************************************/

//                                                                              BASE IO CLASS

/**********************************************************************************************/


// this would be the base class of Digital and Analog IOs
class DECL IODevice : public I2CDEVICE
{
public:
   IODevice(BYTE AddrOfDevice = 0x04, BYTE AddrOfI2C = 0x02);
   virtual ~IODevice();
   //virtual void Notify(const I2CMSG& Msg);

public:
   // Commands for IO device - as specified by the IO board documentation
   I2CCODE CmdTest(BYTE& Status);
   I2CCODE CmdReset()  {return Tx(RESET_OPCODE);};
   I2CCODE CmdSendUnitID(TCHAR SerNo[6], BYTE& ModNo, BYTE& Status);
   I2CCODE CmdReadType(LPTSTR Type);
   I2CCODE CmdChangeAddr(LPTSTR IDcode, BYTE NewAddr);
   I2CCODE CmdRun();
   I2CCODE CmdStop();
   I2CCODE CmdReadReg(BYTE RegAddr, BYTE& RegContents);
   I2CCODE CmdWriteReg(BYTE RegAddr, BYTE Data, BYTE Mask);
   I2CCODE CmdChangeDestinationAddr(BYTE NewAddr);
   I2CCODE CmdClearStatus()    {return Tx(CLEARSTATUS_OPCODE);};
   I2CCODE CmdWriteModuleNo(BYTE NewModNo, BYTE& RetModNo, BYTE& Status);
   I2CCODE CmdAuxOn();
   I2CCODE CmdAuxOff();

public:
   // functions that manipulate class's variables
   BYTE GetI2CAddr() {return I2CAddr;};                                        // returns MasterAddr
   void SetI2CAddr(int Addr)       {I2CAddr = static_cast<BYTE>(Addr);};                   // changes MasterAddr
   BYTE GetDeviceAddr() {return DeviceAddr;};                          // returns DeviceAddr
   void SetDeviceAddr(int Addr)    {DeviceAddr = static_cast<BYTE>(Addr);};        // changes DeviceAddr

public:
   virtual const TCHAR * GetCmdStr(BYTE CmdOpCode);        // returns the string name of the cmd

protected:
   // variables that define the i2c device
   BYTE            I2CAddr;                        // address of I2C
   BYTE            DeviceAddr;                     // address of IO board on comm port
   UINT32          DeviceIDCode;                   // IO board's unique ID code

public:
   // these functions handle requests to start and stop sampling, and also AuxOn/Off
   void StartRequest(void);
   bool StopRequest(void);
   bool MasterStart(void);
   bool MasterStop(void);
   bool AuxOnRequest(void);
   bool AuxOffRequest(void);
   bool MasterAuxOn(void);
   bool MasterAuxOff(void);

public:
   I2CCODE MakeSureModuleIsRunning(void);          // sends CmdRun to the module unless it is already running
   I2CCODE MakeSureModuleIsStopped(void);          // sends CmdStop to the module if it is running


protected:
   int start_requests;                             // how many modules asked for start - see MasterStart/Stop
   int aux_requests;                               // how many modules asked for power
   bool IsRunning;                                 // keeps track of whether the module is running
   bool IsAuxOn;                                   // keeps track

protected:
   // functions that call on the I2C manager to send and receive data
   I2CCODE Tx(BYTE OpCode, PVOID Data = NULL, int LengthOfData = 0, DWORD Timeout=5000);// timeout kludge
   I2CCODE TxRx(I2CMSG& Msg, BYTE OpCode, PVOID Data = NULL, int LengthOfData = 0, DWORD Timeout=5000);// rx data put in Msg param

private:
   CRITICAL_SECTION AccessLock;

public:
   void I2CMSGtoString(I2CMSG& msg, TCHAR* str, bool verbose = false);     // converts an I2CMSG to a string of data (as it should be when sent on the I2C bus) if verbose is false
   // if verbose is true, method will put extra explanatory text into the resulting string
   static void I2CCODEtoString(I2CCODE code, TCHAR* str, bool verbose = false);    // converts an enum I2CCODE to a text string

   bool Running() {return IsRunning;};

public:
   void LockConfig();
   void FreeConfig();

protected:
   I2CCODE MakeSureRunning();

private:
   void HealthMonitor(I2CCODE Result);
   bool ErrorReporting;
   bool StatusReporting;
   DWORD TimeOfLastI2C;
   int FailureCount;
};

#endif
