// LogDesc.h: interface for the CLogDesc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_)
#define AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "..\engine\engine.h"
#include "..\engine\xmlsetup.h"
#include "..\logger\logger.h"

#undef DECL
#ifdef LOGMGR_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

//used to hold cached data
typedef struct
{
   TCHAR Sensor[129];
   TCHAR Units[129];
   TCHAR Data[1001];
   TCHAR Time[129];
   TCHAR Quality[129];
   TCHAR Date[129];

} LVDATAITEM;
typedef LVDATAITEM * PLVDATAITEM;


////////////////////////////////////////////////////////////////////////////
//this is a class that wraps around the LOG class and allows
//us to track the name, size, and wrap flag for UI purposes
class DECL CLogDesc
{
public:
#ifdef TESTLOG
   void TestLog(CListCtrl * pListCtrl);   //test only
#endif
   void EraseData();          //removes all entries from log
   LOGCURSOR CreateCursor();  //creates a LOGCURSOR object
   bool DumpLog();            //sends log data to serial port
   CLogDesc();                //default constructor
   CLogDesc(TCHAR * lpszName, int Size, bool Wrap, bool IgnoreBadData);
   CLogDesc(LPCTSTR lpszName, int Size, bool Wrap, bool IgnoreBadData);
   ~CLogDesc();
   bool Create();             //creates the actual log file
   LPTSTR GetName(){return name;};
   void SetName(LPCTSTR lpszName) {lstrcpy(name, lpszName);};
   int GetSize(){return size;};
   bool IsWrap(){return wrap;};
   bool IsIgnoreBadData() {return ignorebaddata;};
   // Start and end times are ignored when 0 (results in whole log).
   bool ExportLog(LPCTSTR szFile, int iFormat, DWORD Socket = SOCKET_ERROR, 
         TTime tStart = 0, TTime tEnd = 0);

   enum enumExportFormat {COMMA_DELIMITED, TAB_DELIMITED, 
         FIXED_FORMAT, NUM_EXPORT_FORMATS};
   enum enumExportSpan {WHOLE_LOG, SINCE_LAST_EXPORT, 
         START_TIME_TO_END_TIME, START_TIME_TO_END_LOG, 
         START_LOG_TO_END_TIME, NUM_SPAN_OPTIONS};
   TTime m_tLastExportEnd;
   bool m_bAddHeaderOnExport;
   bool m_bReplaceBadValOnExport;
   CString m_strBadValOnExport;
   bool m_bRecordIDBeforeDate;

   bool ContainsNotes();

   LPCTSTR GetColumnTitle(int Column);

   //to edit properties of log from control panel tree
   int EditProperties(CWnd * pParentWnd, bool bNew = false);
   HTREEITEM hItem;           //handle to this item in control panel tree

   bool InUse();
   BOOL WriteSetup(CXMLSetup& XMLSetup);
   BOOL ReadSetup(CXMLSetup& XMLSetup, CXMLToken& XMLToken);

   //ways to write sensor data to log
   BOOL AppendSensor(CString Sensor, CSensorData& data);
   bool Open();
   LOG log;                   //the log file object itself - see logger.h
   UINT32 ChangeCounter;      //used by logger to track changes

   bool AppendNote(LPCTSTR szNote);
private:
   TCHAR name[256];           //log name
   int size;                  //size of file
   bool wrap;                 //true if it should wrap
   bool ignorebaddata;        //true if we should ignore bad data
};

#endif // !defined(AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_)
