// WatchdogTimer.h : header file for WatchdogTimer.cpp.
//

#if !defined(_WATCHDOG_TIMER_INCLUDED_)
#define _WATCHDOG_TIMER_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


/////////////////////////////////////////////////////////////////////////////
// Defines used by TWatchdogTimer class
// These must match those in Xpert/io.h. They are repeated here so that 
// the class can be strictly inline with no outside dependencies.
//
#ifndef WDOG_NO_DFLT_ACTION
#define WDOG_NO_DFLT_ACTION         0
#define WDOG_KILL_PROCESS           1
#define WDOG_RESET_DEVICE           2
extern "C" HANDLE _declspec(dllimport) CreateWatchDogTimer (
    LPCWSTR pszWatchDogName, 
    DWORD dwPeriod, 
    DWORD dwWait, 
    DWORD dwDfltAction, 
    DWORD dwParam,
    DWORD dwFlags);
extern "C" HANDLE _declspec(dllimport)  OpenWatchDogTimer (LPCWSTR pszWatchDogName, DWORD dwFlags);
extern "C" BOOL _declspec(dllimport)  StartWatchDogTimer (HANDLE hWatchDog, DWORD dwFlags);
extern "C" BOOL _declspec(dllimport)  StopWatchDogTimer (HANDLE hWatchDog, DWORD dwFlags);
extern "C" BOOL _declspec(dllimport)  RefreshWatchDogTimer (HANDLE hWatchDog, DWORD dwFlags);
#endif


/////////////////////////////////////////////////////////////////////////////
// TWatchdogTimer class
//
class TWatchdogTimer
{
   ////////////////////////////////
   // Interface
   ////////////////////////////////
   public:
   
   // Constructors and destructor.
   // pszWatchDogName: Watchdog name, where "" is treated as named dog. 
   // dwPeriod: MSec to allow w/o refresh before taking default action. 
   // dwWait: Pause in msec after expiration of period w/o refresh. 
   // dwDfltAction: Action to take if period expires w/o refresh.
   // dwParam: Parameter to be passed to IOCTL_HAL_REBOOT.
   // dwFlags: Reserved, must be 0. 
   TWatchdogTimer(
      LPCTSTR pszWatchDogName = _T("Reboot60Sec"), 
      DWORD dwPeriod = 60000,
      DWORD dwWait = 0,
      DWORD dwDfltAction = WDOG_RESET_DEVICE,
      DWORD dwParam = NULL,
      DWORD dwFlags = 0)
   {
      m_hWDT = NULL;
      m_bIsRunning = false;
      #ifdef _WIN32_WCE
      m_hWDT = CreateWatchDogTimer(pszWatchDogName, dwPeriod, dwWait, dwDfltAction, dwParam, dwFlags);
      #endif
   };
   virtual ~TWatchdogTimer() 
   {
      Stop(); 
      Close();
   };

   // Start the watchdog count down. 
   bool Start() 
   {
      #ifdef _WIN32_WCE
      if (m_hWDT && !m_bIsRunning)
         if (StartWatchDogTimer(m_hWDT, 0))
            m_bIsRunning = true;
      #endif
      return m_bIsRunning;
   };

   // Refresh the watchdog timer.
   bool Refresh()
   {
      #ifdef _WIN32_WCE
      if (m_hWDT && m_bIsRunning)
         return RefreshWatchDogTimer(m_hWDT, 0) ? true : false;
      #endif
      return false;
   };
   
   // Stop the watchdog count down. 
   bool Stop() 
   {
      #ifdef _WIN32_WCE
      if (m_hWDT && m_bIsRunning)
         if (StopWatchDogTimer(m_hWDT, 0))
            m_bIsRunning = false;
      #endif
      return !m_bIsRunning;
   };

   // Close watchdog timer. 
   void Close() 
   {
      Stop();
      if (m_hWDT) 
         CloseHandle(m_hWDT);
   };

   // Indicates whether watchdog is up and running. 
   bool IsRunning() {return m_bIsRunning;};
   
   ////////////////////////////////
   // Implementation
   ////////////////////////////////
   protected:

   // Handle to watchdog. 
   HANDLE m_hWDT;
   bool m_bIsRunning;
};


#endif // !defined(_WATCHDOG_TIMER_INCLUDED_)
