#ifndef _SOCKET_H

#define _SOCKET_H

#include <Winsock2.h>
#include "ssp.h"

#undef DECL
#if defined(NOT_XPERT)
#define DECL
#else
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif
#endif


const int SOCKET_NULL = SOCKET_ERROR;

enum TSocketType {ST_TCP, ST_UDP};

class DECL TThread
{
public:
   TThread(int RememberToCallRun);
   virtual ~TThread();
   virtual void ThreadLoop() {};
   bool Run();
   bool Stop();
   HANDLE hStopThread;
   HANDLE hThreadStopped;
   bool IsDone();
   bool m_bAutoDelete;
private:
   bool m_bDone;
   static UINT BootStrap(TThread* pThread);
};


class DECL TSocket
{
public:
   TSocket(TSocketType stSocketType=ST_TCP);
   TSocket(SOCKET aSocket);
   virtual ~TSocket();
// Character Read/Write with Timeouts
   int Read(char* Buffer, int MaxLen, int Timeout=0);
   int Write(const char* Buffer, int Len, int Timeout=0);
   int Write(CString Buffer, int Timeout=0);
// SSP Read/Write
   bool Read(TSSPData& SSPData, int Timeout);
   bool Write(TSSPData& SSPData, int Timeout);
//
   void DisableBuffer();
   int Abort();
   int Reset();
   int Close(bool Gracefull=false);
   bool Closed();
   bool ReadyToWrite(DWORD Timeout);
   int GetInputCount(DWORD Timeout = 0);
   SOCKET SetSocket(SOCKET NewSocket);
   SOCKET GetSocket();
   void FlushInput();
   bool GetSockName(CString& Address, int& Port);
   HANDLE GetDataReadyEventFlag();
   bool IsUDP();
   bool GetConnectName(CString& Address, int& Port);
   int Connect(LPCTSTR Addr, int Port, TSocketType stSocketType=ST_TCP);
   bool IsConnected();
   bool IsConnectedLocal();
private:
   friend class TSocketClient;
   friend class TSocketServer;
   SOCKET Socket;
   HANDLE hDataReady;
   bool bInitDataReady;
   bool UDP;
   SOCKADDR_IN SockAddr;
   CRITICAL_SECTION Lock;
};

class DECL TServerThread : public TThread
{
public:
   TServerThread(TSocketServer& ASocketServer, int ASocket, int RememberToCallRun);
   virtual ~TServerThread();
   void ThreadLoop();
   bool IsDone();
public:
   TSocket Socket;
   HANDLE RequestLoan;
   HANDLE GrantLoan;
   HANDLE DoneLoan;
   bool LoanApproved;
   bool Done;
private:
   friend class TSocketServer;
   TSocketServer& SocketServer;
};

class DECL TSocketServer
{
public:
   TSocketServer(TSocketType stSocketType=ST_TCP);
   virtual ~TSocketServer();
   virtual void ServerThread(TServerThread& Thread, TSocket& Socket) {};
   virtual TServerThread* CreateServerThread(int ASocket);
   virtual void PrepareToRun(TServerThread* Thread); 
   int StartServer(int aPort);
   int StopServer();
   int GetPort();
   bool IsDone();
   bool IsUDP();
public:
   CPtrArray ServerList;
   CRITICAL_SECTION Lock;
   void FreezeServerList();
   void UnFreezeServerList();
   int Port;
private:
   int FreezeCount;
   static UINT BootStrap(TSocketServer* pSocketServer);
   friend class TServerThread;
   TSocket ListenSocket;
   HANDLE hListenThread;
   void ListenThread();
   bool Stopping;
   bool Done;
   bool UDP;
   SOCKADDR_IN SockAddr;
};

class DECL TSocketClient : public TSocket
{
public:
   TSocketClient();
   virtual ~TSocketClient();
};


/*
   TSocketClient Test Code
   -----------------------

   char Buffer[10000];
   TSocketClient Client;
   int ConRes = Client.Connect(_T("www.ibm.com"), 80);
   int WError = WSAGetLastError();
   int WrRes = Client.Write("GET /index.html\r\n", 17, 5000);
   int ReadLen = Client.Read(&Buffer[0], 10000, 5000);
   Client.Close();

*/

CString GetIPAddress();

#endif // _SOCKET_H